<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://welaunch.io/plugins/woocommerce-reward-points/
 * @since      1.0.0
 *
 * @package    WooCommerce_delivery
 * @subpackage WooCommerce_delivery/public
 */
class WooCommerce_Ultimate_Pricing_Cart_Fees extends WooCommerce_Ultimate_Pricing {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	protected $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of this plugin.
	 */
	protected $version;

	/**
	 * options of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      array    $options
	 */
	protected $options;


	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) 
	{
		$this->plugin_name = $plugin_name;
		$this->version = $version;
	}
	
	public function init()
	{

		global $woocommerce_ultimate_pricing_options;
		$this->options = $woocommerce_ultimate_pricing_options;
		$this->add_to_cart_running = false;

		if (!$this->get_option('enable')) {
			return false;
		}

		if (!$this->get_option('cartFeesEnabled')) {
			return false;
		}

		$this->user_id = get_current_user_id();

		add_action( 'woocommerce_cart_calculate_fees', array($this, 'add_cart_fees') );
		add_action( 'woocommerce_cart_totals_get_fees_from_cart_taxes', array($this, 'excludeCartFeesTaxes'), 10 ,3);
	}

    public function add_cart_fees()
    {
		$percentageFeeTotal = 0;
		$fixedFeeTotal = 0;

		$cart = WC()->cart;
		$cartItems = $cart->get_cart();
		if(empty($cartItems)) {
			return;
		}
		
		$role = $this->get_user_role($this->user_id);
		$cartItemsCount = count($cartItems);
		$cartQuantity = array_sum($cart->get_cart_item_quantities());
		$cartSubtotal = floatval( $cart->get_subtotal() );
		if($this->get_option('cartFeesSubotalWithTaxes')) {
			$cartSubtotal = $cartSubtotal + floatval( $cart->get_subtotal_tax() );
		}

		if(class_exists('WooCommerce_Reward_Points')) {
			$userCurrentLevel = 1;
			if($this->user_id > 0) { 
				$currentLevelMeta = get_user_meta($this->user_id, 'woocommerce_reward_points_current_level', true);
				if($currentLevelMeta) {
					$userCurrentLevel = (int) $currentLevelMeta;
				}
			} else {
				$currentLevelMeta = isset($_SESSION['woocommerce_reward_points_current_level']) ? $_SESSION['woocommerce_reward_points_current_level'] : false;
				if($currentLevelMeta) {
					$userCurrentLevel = (int) $currentLevelMeta;
				}
			}
		}

		$B2BKingCustomergroup = get_user_meta($this->user_id, 'b2bking_customergroup', true);

		// Always zero, cause not calculated yet
		$cartTotal = $cart->get_total();

		$cartFeesCount = $this->get_option('cartFeesCount');
		for ($i=1; $i <= $cartFeesCount; $i++) { 

			if(!$this->get_option('cartFeesRule' . $i . 'Enabled')) {
				continue;
			}

		    $roles = $this->get_option('cartFeesRule' . $i . 'Roles');
		    if(!empty($roles)) {
			   		
			   	// No Role or Guest
			   	if(!$role) {
			   		continue;
			   	}

				if(!in_array($role, $roles)) {
					continue;
				}
		    }

		    $B2BKingCustomerGroups = $this->get_option('cartFeesRule' . $i . 'B2BKingCustomerGroups');
		    if(!empty($B2BKingCustomerGroups)) {

			   	if(!in_array($B2BKingCustomergroup, $B2BKingCustomerGroups)) {
			   		continue;
		   		}
		    }

		    $dateRange = $this->get_option('cartFeesRule' . $i . 'DateRange');
		    if(!empty($dateRange)) {
		    	$dateRange = explode(' to ', $dateRange);
		    	$startDate = $dateRange[0];
		    	$endDate = $dateRange[1];
				if(!empty($startDate) && !empty($endDate)) {
					$currentDateObject = new DateTime();
					$startDateObject = new DateTime($startDate);
					$endDateObject = new DateTime($endDate . ' 23:59');
				    if($currentDateObject < $startDateObject || $currentDateObject > $endDateObject) {
				        continue;
				    }
				}
		    }

		    $level = (int) $this->get_option('cartFeesRule' . $i . 'Level');
			if($level > 0 && class_exists('WooCommerce_Reward_Points')) {
				if($userCurrentLevel < $level) {
					continue;
				}
			}

		    $reviews = $this->get_option('cartFeesRule' . $i . 'Reviews');
		    if(!empty($reviews)) {
			    $userReviews = $this->get_user_review_count($this->user_id);
			    if($userReviews < $reviews) {
			    	continue;
			    }
		    }

		    $orders = $this->get_option('cartFeesRule' . $i . 'Orders');
		    if(!empty($orders)) {
			    $userOrders = $this->get_user_order_count($this->user_id);
			    if($userOrders < $orders) {
			    	continue;
			    }
		    }

		    $cartSubtotalCheck = floatval( $this->get_option('cartFeesRule' . $i . 'CartSubtotal') );
		    if(!empty($cartSubtotalCheck)) {
			    if($cartSubtotal < $cartSubtotalCheck) {
			    	continue;
			    }
		    }

		    $cartTotalCheck = $this->get_option('cartFeesRule' . $i . 'CartTotal');
		    if(!empty($cartTotalCheck)) {
			    if($cartTotal < $cartTotalCheck) {
			    	continue;
			    }
		    }

			$categories = $this->get_option('cartFeesRule' . $i . 'Categories');
			$products = $this->get_option('cartFeesRule' . $i . 'Products');
			$tags = $this->get_option('cartFeesRule' . $i . 'Tags');

			if(!empty($categories) || !empty($products) || !empty($tags)) {

				$applys = false;
				$appliedSubtotalWithTaxes = 0;
				$appliedSubtotalWithoutTaxes = 0;
			    foreach ($cartItems as $cartItem) {
			    	
			    	$cartItemProductId = $cartItem['product_id'];
			    	$cartItemQuantiy = $cartItem['quantity'];
			    	
			    	$cartItemProduct = wc_get_product($cartItemProductId);
			    	if(!$cartItemProduct) {
			    		continue;
			    	}

					$cartItemProductIdToCheckTaxonomies = $cartItemProductId;
					if($cartItemProduct->is_type('variation')) {
						$cartItemProductIdToCheckTaxonomies = $cartItemProduct->get_parent_id();
					}

				    $categories = $this->get_option('cartFeesRule' . $i . 'Categories');
				    if(!empty($categories)) {
					    if( !has_term($categories, 'product_cat', $cartItemProductIdToCheckTaxonomies) ) {
					    	// remove Quantity & Count
					    	$cartItemsCount = $cartItemsCount - 1;
					    	$cartQuantity = $cartQuantity - $cartItemQuantiy;
					        continue;
					    }
				    }

				    $products = $this->get_option('cartFeesRule' . $i . 'Products');
				    if(!empty($products)) {
					    if( !in_array($cartItemProductIdToCheckTaxonomies, $products)) {
					    	// remove Quantity & Count
					    	$cartItemsCount = $cartItemsCount - 1;
					    	$cartQuantity = $cartQuantity - $cartItemQuantiy;
					        continue;
					    }
				    }

				    $tags = $this->get_option('cartFeesRule' . $i . 'Tags');
				    if(!empty($tags)) {
					    if( !has_term($tags, 'product_tag', $cartItemProductIdToCheckTaxonomies) ) {
					    	// remove Quantity & Count
					    	$cartItemsCount = $cartItemsCount - 1;
					    	$cartQuantity = $cartQuantity - $cartItemQuantiy;
					        continue;
					    }
				    }

					$appliedSubtotalWithTaxes = $appliedSubtotalWithTaxes + $cartItem['line_subtotal'] + $cartItem['line_subtotal_tax'];
					$appliedSubtotalWithoutTaxes = $appliedSubtotalWithoutTaxes + $cartItem['line_subtotal'];

				    $applys = true;
			    }

			    if(!$applys) {
			    	continue;
			    }
		    }

		    $cartItemCountCheck = $this->get_option('cartFeesRule' . $i . 'CartItemCount');
		    if(!empty($cartItemCountCheck)) {
			    if($cartItemsCount < $cartItemCountCheck) {
			    	continue;
			    }
		    }


		    $cartQuantityCount = $this->get_option('cartFeesRule' . $i . 'CartQuantityCount');
		    if(!empty($cartQuantityCount)) {
			    if($cartQuantity < $cartQuantityCount) {
			    	continue;
			    }
		    }

		    global $newTotal;

	    	$discount = 0;
	    	$newTotal = 0;

			if($this->get_option('cartFeesRule' . $i . 'Type') == "percentage") {
				$discount = $cartSubtotal * ($this->get_option('cartFeesRule' . $i . 'PercentageFee') / 100);
			} elseif($this->get_option('cartFeesRule' . $i . 'Type') == "fixedTotal") {
				$newTotal = $this->get_option('cartFeesRule' . $i . 'FixedTotal');
			} elseif($this->get_option('cartFeesRule' . $i . 'Type') == "percentageTotalAppliedWithTax") {
				$discount = $appliedSubtotalWithTaxes * ($this->get_option('cartFeesRule' . $i . 'PercentageTotalAppliedWithTaxPercentage') / 100);
			} elseif($this->get_option('cartFeesRule' . $i . 'Type') == "percentageTotalAppliedWithoutTax") {
				$discount = $appliedSubtotalWithoutTaxes * ($this->get_option('cartFeesRule' . $i . 'PercentageTotalAppliedWithoutTaxPercentage') / 100);
			} else {
				$discount = $this->get_option('cartFeesRule' . $i . 'FixedFee');
			}

			if($discount > 0) {
				$cart->add_fee($this->get_option('cartFeesRule' . $i . 'Title') , $discount, false, '');
			}

			if($this->get_option('cartFeesRule' . $i . 'Type') == "fixedTotal" && $newTotal > 0) {
				add_filter( 'woocommerce_calculated_total', function( $total, $cart ) {
						global $newTotal;
					   	return $newTotal;
					}, 10, 2 );
			}

		}
	}

	public function excludeCartFeesTaxes($taxes, $fee, $cart)
	{
		if(empty($taxes)) {
			return $taxes;
		}

		if(!$this->get_option('cartFeesExcludeTaxes')) {
			return $taxes;
		}

		foreach ($taxes as $taxKey => $tax) {
			if($tax <= 0) {
				unset($taxes[$taxKey]);
			}
		}
		
	    return $taxes;
	}
}